!> @file user_3d_data_averaging.f90
!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_3d_data_averaging.f90 1683 2015-10-07 23:57:51Z knoop $
!
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
! 
! 1353 2014-04-08 15:21:23Z heinze
! REAL constants provided with KIND-attribute
!
! 1322 2014-03-20 16:38:49Z raasch
! REAL functions provided with KIND-attribute
!
! 1320 2014-03-20 08:40:49Z raasch
! kind-parameters added to all INTEGER and REAL declaration statements,
! kinds are defined in new module kinds,
! revision history before 2012 removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 211 2008-11-11 04:46:24Z raasch
! Former file user_interface.f90 split into one file per subroutine
!
! Description:
! ------------
!> Sum up and time-average user-defined output quantities as well as allocate
!> the array necessary for storing the average.
!------------------------------------------------------------------------------!
 SUBROUTINE user_3d_data_averaging( mode, variable )
 

    USE control_parameters

    USE indices

    USE kinds

    USE user

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  mode    !< 
    CHARACTER (LEN=*) :: variable !< 

    INTEGER(iwp) ::  i !< 
    INTEGER(iwp) ::  j !< 
    INTEGER(iwp) ::  k !< 
    INTEGER(iwp) ::  m !< 

    INTEGER(iwp) ::  dummy !< dummy variable

    IF ( mode == 'allocate' )  THEN

       SELECT CASE ( TRIM( variable ) )

!
!--       Uncomment and extend the following lines, if necessary.
!--       The arrays for storing the user defined quantities (here u2_av) have
!--       to be declared and defined by the user!
!--       Sample for user-defined output:
          CASE ( 'dd' )
!              IF ( .NOT. ALLOCATED( dd_av ) )  THEN
!                 ALLOCATE( dd_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
!              ENDIF
!              dd_av = 0.0_wp
               dummy = 0

          CASE ( 'ff' )
             IF ( .NOT. ALLOCATED( ff_av ) )  THEN
                ALLOCATE( ff_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             ff_av = 0.0_wp

          CASE ( 'ff_r' )
             IF ( .NOT. ALLOCATED( ff_r_av ) )  THEN
                ALLOCATE( ff_r_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             ff_r_av = 0.0_wp

          CASE ( 'ti' )
             IF ( .NOT. ALLOCATED( ti_av ) )  THEN
                ALLOCATE( ti_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
             ENDIF
             ti_av = 0.0_wp

         CASE ( 'u2' )
            IF ( .NOT. ALLOCATED( u2_av ) )  THEN
               ALLOCATE( u2_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
            ENDIF
            u2_av = 0.0_wp

         CASE ( 'ui' )
            IF ( .NOT. ALLOCATED( ui_av ) )  THEN
               ALLOCATE( ui_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
            ENDIF
            ui_av = 0.0_wp

         CASE ( 'uvgs' )
            IF ( .NOT. ALLOCATED( uvgs_av ) )  THEN
               ALLOCATE( uvgs_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
            ENDIF
            uvgs_av = 0.0_wp

         CASE ( 'v2' )
            IF ( .NOT. ALLOCATED( v2_av ) )  THEN
               ALLOCATE( v2_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
            ENDIF
            v2_av = 0.0_wp

         CASE ( 'vi' )
            IF ( .NOT. ALLOCATED( vi_av ) )  THEN
               ALLOCATE( vi_av(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
            ENDIF
            vi_av = 0.0_wp

          CASE DEFAULT
             CONTINUE

       END SELECT

    ELSEIF ( mode == 'sum' )  THEN

       SELECT CASE ( TRIM( variable ) )

!
!--       Uncomment and extend the following lines, if necessary.
!--       The arrays for storing the user defined quantities (here u2 and
!--       u2_av) have to be declared and defined by the user!
!--       Sample for user-defined output:
          CASE ( 'dd' )
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    DO  k = nzb, nzt
!                       dd_av(k,j,i) = dd_av(k,j,i) + dd(k,j,i)
!                    ENDDO
!                 ENDDO
!              ENDDO
               dummy = 0

          CASE ( 'ff' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt
                      ff_av(k,j,i) = ff_av(k,j,i) + ff(k,j,i)
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'ff_r' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt
                      ff_r_av(k,j,i) = ff_r_av(k,j,i) + ff_r(k,j,i)
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'ti' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt
                      ti_av(k,j,i) = ti_av(k,j,i) + ti(k,j,i)
                   ENDDO
                ENDDO
             ENDDO

         CASE ( 'u2' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     u2_av(k,j,i) = u2_av(k,j,i) + u2(k,j,i)
                  ENDDO
               ENDDO
            ENDDO

         CASE ( 'ui' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     ui_av(k,j,i) = ui_av(k,j,i) + ui(k,j,i)
                  ENDDO
               ENDDO
            ENDDO

        CASE ( 'uvgs' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     uvgs_av(k,j,i) = uvgs_av(k,j,i) + uvgs(k,j,i)
                  ENDDO
               ENDDO
            ENDDO

         CASE ( 'v2' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     v2_av(k,j,i) = v2_av(k,j,i) + v2(k,j,i)
                  ENDDO
               ENDDO
            ENDDO

         CASE ( 'vi' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     vi_av(k,j,i) = vi_av(k,j,i) + vi(k,j,i)
                  ENDDO
               ENDDO
            ENDDO

          CASE DEFAULT
             CONTINUE

       END SELECT

    ELSEIF ( mode == 'average' )  THEN

       SELECT CASE ( TRIM( variable ) )

!
!--       Uncomment and extend the following lines, if necessary.
!--       The arrays for storing the user defined quantities (here u2_av) have
!--       to be declared and defined by the user!
!--       Sample for user-defined output:
          CASE ( 'dd' )
          ! wind direction will be calculated in 'user_data_output_3d/2d' from
          ! u_av and v_av
!              DO  i = nxlg, nxrg
!                 DO  j = nysg, nyng
!                    DO  k = nzb, nzt
!                       dd_av(k,j,i) = dd_av(k,j,i) / REAL( average_count_3d, KIND=wp )
!                    ENDDO
!                 ENDDO
!              ENDDO
               dummy = 0

          CASE ( 'ff' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt
                      ff_av(k,j,i) = ff_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'ff_r' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt
                      ff_r_av(k,j,i) = ff_r_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

          CASE ( 'ti' )
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb, nzt
                      ti_av(k,j,i) = ti_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                   ENDDO
                ENDDO
             ENDDO

         CASE ( 'u2' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     u2_av(k,j,i) = u2_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                  ENDDO
               ENDDO
            ENDDO

         CASE ( 'ui' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     ui_av(k,j,i) = ui_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                  ENDDO
               ENDDO
            ENDDO

         CASE ( 'uvgs' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     uvgs_av(k,j,i) = uvgs_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                  ENDDO
               ENDDO
            ENDDO

         CASE ( 'v2' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     v2_av(k,j,i) = v2_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                  ENDDO
               ENDDO
            ENDDO

         CASE ( 'vi' )
            DO  i = nxlg, nxrg
               DO  j = nysg, nyng
                  DO  k = nzb, nzt+1
                     vi_av(k,j,i) = vi_av(k,j,i) / REAL( average_count_3d, KIND=wp )
                  ENDDO
               ENDDO
            ENDDO

       END SELECT

    ENDIF


 END SUBROUTINE user_3d_data_averaging
