!> @file user_check_data_output.f90
!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_check_data_output.f90 1683 2015-10-07 23:57:51Z knoop $
!
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
! 
! 1320 2014-03-20 08:40:49Z raasch
! kind-parameters added to all INTEGER and REAL declaration statements,
! kinds are defined in new module kinds,
! revision history before 2012 removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 211 2008-11-11 04:46:24Z raasch
! Former file user_interface.f90 split into one file per subroutine
!
! Description:
! ------------
!> Set the unit of user defined output quantities. For those variables
!> not recognized by the user, the parameter unit is set to "illegal", which
!> tells the calling routine that the output variable is not defined and leads
!> to a program abort.
!------------------------------------------------------------------------------!
 SUBROUTINE user_check_data_output( variable, unit )
 

    USE kinds

    USE user

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  unit     !< 
    CHARACTER (LEN=*) ::  variable !<


    SELECT CASE ( TRIM( variable ) )
!--    quantities with suffix '_b*' are defined to be at the first grid point
!--    above topography

!--    wind direction of horizontal wind
       CASE ( 'dd', 'dd_b*' )
          unit = 'deg'

!--    wind speed (3D)
       CASE ( 'ff', 'ff_b*' )
          unit = 'm/s'

!--    velocity ratio
       CASE ( 'ff_r', 'ff_r_b*' )
          unit = ' '

!--    scalar concentration
       CASE ( 's_b*' )
          unit = 'kg/m3'

!--    turbulence intensity
       CASE ( 'ti', 'ti_b*' )
          unit = '1/s'

!--    horizontal wind speed component
       CASE ( 'u_b*', 'v_b*' )
          unit = 'm/s'

!-------------------------------------
!--    Output needed for gust analysis

       CASE ( 'u2', 'u2_av', 'v2', 'v2_av' )
          unit = 'm2/s2'

       CASE ( 'ui', 'ui_av', 'vi', 'vi_av' )
          unit = 'm/s'

       CASE ( 'uvgs', 'uvgs_av' )
          unit = 'm/s'

       CASE ( 'uvmax' )
          unit = 'm/s'

       CASE ( 'gust11', 'gust21', 'gust31', 'gust41', 'gust51', 'gust61',      &
              'gust12', 'gust22', 'gust32', 'gust42', 'gust52', 'gust62',      &
              'gust13', 'gust23', 'gust33', 'gust43', 'gust53', 'gust63' )
          unit = ''

       CASE ( 'ros011', 'ros021', 'ros031', 'ros041', 'ros051', 'ros061',      &
              'ros071', 'ros081', 'ros091', 'ros101', 'ros111', 'ros121',      &
              'ros012', 'ros022', 'ros032', 'ros042', 'ros052', 'ros062',      &
              'ros072', 'ros082', 'ros092', 'ros102', 'ros112', 'ros122',      &
              'ros013', 'ros023', 'ros033', 'ros043', 'ros053', 'ros063',      &
              'ros073', 'ros083', 'ros093', 'ros103', 'ros113', 'ros123',      &
              'ros014', 'ros024', 'ros034', 'ros044', 'ros054', 'ros064',      &
              'ros074', 'ros084', 'ros094', 'ros104', 'ros114', 'ros124',      &
              'ros015', 'ros025', 'ros035', 'ros045', 'ros055', 'ros065',      &
              'ros075', 'ros085', 'ros095', 'ros105', 'ros115', 'ros125',      &
              'ros016', 'ros026', 'ros036', 'ros046', 'ros056', 'ros066',      &
              'ros076', 'ros086', 'ros096', 'ros106', 'ros116', 'ros126' )
          unit = ''

       CASE DEFAULT
          unit = 'illegal'

    END SELECT


 END SUBROUTINE user_check_data_output

