!> @file user_data_output_2d.f90
!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_data_output_2d.f90 1683 2015-10-07 23:57:51Z knoop $
!
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
! 
! 1551 2015-03-03 14:18:16Z maronga
! Replaced nzb and nzt+1 with the new array bounds nzb_do and nzt_do.
! 
! 1320 2014-03-20 08:40:49Z raasch
! kind-parameters added to all INTEGER and REAL declaration statements,
! kinds are defined in new module kinds,
! revision history before 2012 removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 211 2008-11-11 04:46:24Z raasch
! Former file user_interface.f90 split into one file per subroutine
!
! Description:
! ------------
!> Resorts the user-defined output quantity with indices (k,j,i) to a
!> temporary array with indices (i,j,k) and sets the grid on which it is defined.
!> Allowed values for grid are "zu" and "zw".
!------------------------------------------------------------------------------!
 SUBROUTINE user_data_output_2d( av, variable, found, grid, local_pf, two_d, nzb_do, nzt_do )
 
    USE arrays_3d

    USE averaging

    USE constants,                                                             &
        ONLY:  pi

    USE indices

    USE kinds

    USE user

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  grid     !< 
    CHARACTER (LEN=*) ::  variable !< 

    INTEGER(iwp) ::  av !< 
    INTEGER(iwp) ::  i  !< 
    INTEGER(iwp) ::  j  !< 
    INTEGER(iwp) ::  k  !< 
    INTEGER(iwp) ::  nzb_do !< lower limit of the domain (usually nzb)
    INTEGER(iwp) ::  nzt_do !< upper limit of the domain (usually nzt+1)

    LOGICAL      ::  found !< 
    LOGICAL      ::  two_d !< flag parameter that indicates 2D variables (horizontal cross sections)

    REAL(wp)     ::  rad_to_deg !< factor to change radian to degree

    REAL(wp), DIMENSION(nxlg:nxrg,nysg:nyng,nzb:nzt+1) ::  local_pf !< 


    found = .TRUE.

    rad_to_deg = 180.0_wp / pi

    SELECT CASE ( TRIM( variable ) )

!
!--    Uncomment and extend the following lines, if necessary.
!--    The arrays for storing the user defined quantities (here u2 and u2_av)
!--    have to be declared and defined by the user!
!--    Sample for user-defined output:
!       CASE ( 'u2_xy', 'u2_xz', 'u2_yz' )
!          IF ( av == 0 )  THEN
!             DO  i = nxlg, nxrg
!                DO  j = nysg, nyng
!                   DO  k = nzb_do, nzt_do
!                      local_pf(i,j,k) = u2(k,j,i)
!                   ENDDO
!                ENDDO
!             ENDDO
!          ELSE
!             DO  i = nxlg, nxrg
!                DO  j = nysg, nyng
!                   DO  k = nzb_do, nzt_do
!                      local_pf(i,j,k) = u2_av(k,j,i)
!                   ENDDO
!                ENDDO
!             ENDDO
!          ENDIF
!
!          grid = 'zu'

!--    horizontal wind direction
       CASE ( 'dd_xy', 'dd_xz', 'dd_yz' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg-1
                DO  j = nysg, nyng-1
                   DO  k = nzb_do, nzt_do
                      IF ( k .GT. nzb_s_inner(j,i) ) THEN
                         local_pf(i,j,k) = 180.0_wp + rad_to_deg * ATAN2(      &
                                             ( u(k,j,i) + u(k,j,i+1) ) * 0.5,  &
                                             ( v(k,j,i) + v(k,j+1,i) ) * 0.5   )
                      ELSE
                         local_pf(i,j,k) = -200.0_wp
                      ENDIF
                   ENDDO
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg-1
                DO  j = nysg, nyng-1
                   DO  k = nzb_do, nzt_do
                      IF ( k .GT. nzb_s_inner(j,i) ) THEN
                         local_pf(i,j,k) = 180.0_wp + rad_to_deg * ATAN2(      &
                                       ( u_av(k,j,i) + u_av(k,j,i+1) ) * 0.5,  &
                                       ( v_av(k,j,i) + v_av(k,j+1,i) ) * 0.5   )
                      ELSE
                         local_pf(i,j,k) = -200.0_wp
                      ENDIF
                   ENDDO
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu'

!--    horizontal wind direction above surface
       CASE ( 'dd_b*_xy' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg-1
                DO  j = nysg, nyng-1
                   k = nzb_s_inner(j,i)+1
                   local_pf(i,j,nzb+1) = 180.0_wp + rad_to_deg * ATAN2(  &
                      (   u(k,j,i)   + u(k,j,i+1)              &
                        + u(k+1,j,i) + u(k+1,j,i+1) ) * 0.25,  &
                      (   v(k,j,i)   + v(k,j+1,i)              &
                        + v(k+1,j,i) + v(k+1,j+1,i) ) * 0.25   )
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg-1
                DO  j = nysg, nyng-1
                   k = nzb_s_inner(j,i)+1
                   local_pf(i,j,nzb+1) = 180.0_wp + rad_to_deg * ATAN2(        &
                      (   u_av(k,j,i)   + u_av(k,j,i+1)              &
                        + u_av(k+1,j,i) + u_av(k+1,j,i+1) ) * 0.25,  &
                      (   v_av(k,j,i)   + v_av(k,j+1,i)              &
                        + v_av(k+1,j,i) + v_av(k+1,j+1,i) ) * 0.25   )
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu1'
          two_d = .TRUE.

!--    wind speed
       CASE ( 'ff_xy', 'ff_xz', 'ff_yz' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = ff(k,j,i)
                   ENDDO
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = ff_av(k,j,i)
                   ENDDO
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu'

!--    wind speed above surface
       CASE ( 'ff_b*_xy' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * (  &
                         ff(nzb_s_inner(j,i)+1,j,i) &
                       + ff(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * (     &
                         ff_av(nzb_s_inner(j,i)+1,j,i) &
                       + ff_av(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu1'
          two_d = .TRUE.

!--    velocity ratio
       CASE ( 'ff_r_xy', 'ff_r_xz', 'ff_r_yz' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = ff_r(k,j,i)
                   ENDDO
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = ff_r_av(k,j,i)
                   ENDDO
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu'

!--    velocity ratio above surface
       CASE ( 'ff_r_b*_xy' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * (    &
                         ff_r(nzb_s_inner(j,i)+1,j,i) &
                       + ff_r(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * (       &
                         ff_r_av(nzb_s_inner(j,i)+1,j,i) &
                       + ff_r_av(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu1'
          two_d = .TRUE.

!--    scalar concentration above surface
       CASE ( 's_b*_xy' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * ( &
                         q(nzb_s_inner(j,i)+1,j,i) &
                       + q(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5 * (       &
                         s_av(nzb_s_inner(j,i)+1,j,i) &
                       + s_av(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu1'
          two_d = .TRUE.

!--    turbulence intensity
       CASE ( 'ti_xy', 'ti_xz', 'ti_yz' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = ti(k,j,i)
                   ENDDO
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = ti_av(k,j,i)
                   ENDDO
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu'

!--    scalar concentration above surface
       CASE ( 'ti_b*_xy' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5 * (     &
                         ti(nzb_s_inner(j,i)+1,j,i) &
                       + ti(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * (     &
                         ti_av(nzb_s_inner(j,i)+1,j,i) &
                       + ti_av(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu1'
          two_d = .TRUE.

!--    u above surface
       CASE ( 'u_b*_xy' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * ( &
                         u(nzb_s_inner(j,i)+1,j,i) &
                       + u(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * (    &
                         u_av(nzb_s_inner(j,i)+1,j,i) &
                       + u_av(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu1'
          two_d = .TRUE.

!--    v above surface
       CASE ( 'v_b*_xy' )

          IF ( av == 0 )  THEN

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * ( & 
                         v(nzb_s_inner(j,i)+1,j,i) &
                       + v(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ELSE

             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   local_pf(i,j,nzb+1) = .5_wp * (    &
                         v_av(nzb_s_inner(j,i)+1,j,i) &
                       + v_av(nzb_s_inner(j,i)+2,j,i) )
                ENDDO
             ENDDO

          ENDIF

          grid  = 'zu1'
          two_d = .TRUE.

!------------------------------
!--    output for gust analysis

       CASE ( 'uvgs_xy', 'uvgs_xz', 'uvgs_yz' )
          IF ( av == 0 )  THEN
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = uvgs(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ELSE
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = uvgs_av(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ENDIF
          grid = 'zw'

       CASE ( 'u2_xy', 'u2_xz', 'u2_yz' )
          IF ( av == 0 )  THEN
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = u2(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ELSE
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = u2_av(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ENDIF
          grid = 'zw'

       CASE ( 'v2_xy', 'v2_xz', 'v2_yz' )
          IF ( av == 0 )  THEN
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = v2(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ELSE
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = v2_av(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ENDIF
          grid = 'zw'

       CASE ( 'ui_xy', 'ui_xz', 'ui_yz' )
          IF ( av == 0 )  THEN
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = ui(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ELSE
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = ui_av(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ENDIF
          grid = 'zw'

       CASE ( 'vi_xy', 'vi_xz', 'vi_yz' )
          IF ( av == 0 )  THEN
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = vi(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ELSE
             DO  i = nxlg, nxrg
                DO  j = nysg, nyng
                   DO  k = nzb_do, nzt_do
                      local_pf(i,j,k) = vi_av(k,j,i)
                   ENDDO
                ENDDO
             ENDDO
          ENDIF
          grid = 'zw'

       CASE DEFAULT
          found = .FALSE.
          grid  = 'none'

    END SELECT


 END SUBROUTINE user_data_output_2d

