!> @file user_data_output_mask.f90
!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! ------------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_data_output_mask.f90 1683 2015-10-07 23:57:51Z knoop $
!
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
!
! 1320 2014-03-20 08:40:49Z raasch
! kind-parameters added to all INTEGER and REAL declaration statements, 
! kinds are defined in new module kinds, 
! old module precision_kind is removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! Description:
! ------------
!> Resorts the user-defined output quantity with indices (k,j,i) to a
!> temporary array with indices (i,j,k) for masked data output.
!------------------------------------------------------------------------------!
 SUBROUTINE user_data_output_mask( av, variable, found, local_pf )
 
    USE arrays_3d

    USE averaging

    USE constants,                                                             &
        ONLY:  pi

    USE control_parameters

    USE indices
    
    USE kinds
    
    USE user

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  variable   !< 

    INTEGER(iwp) ::  av   !< 
    INTEGER(iwp) ::  i    !< 
    INTEGER(iwp) ::  j    !< 
    INTEGER(iwp) ::  k    !< 

    INTEGER(iwp) ::  ii,jj,kk  !< 
    INTEGER(iwp) ::  mmm,nnn   !< 

    LOGICAL ::  found     !< 

    REAL(wp)     ::  rad_to_deg !< factor to change radian to degree

    REAL(wp),                                                                  &
       DIMENSION(mask_size_l(mid,1),mask_size_l(mid,2),mask_size_l(mid,3)) ::  &
          local_pf   !< 


    found = .TRUE.

    rad_to_deg = 180.0_wp / pi

    SELECT CASE ( TRIM( variable ) )

!--    wind direction
       CASE ( 'dd' )
          IF ( av == 0 )  THEN
             DO  i = 1, mask_size_l(mid,1)
                DO  j = 1, mask_size_l(mid,2)
                   DO  k = 1, mask_size_l(mid,3)
                      kk = mask_k(mid,k)
                      jj = mask_j(mid,j)
                      ii = mask_i(mid,i)
                      IF ( kk .GT. nzb_s_inner(jj,ii) ) THEN
                         local_pf(i,j,k) = 180.0_wp + rad_to_deg * ATAN2(      &
                                       ( u(kk,jj,ii) + u(kk,jj,ii+1) ) * 0.5,  &
                                       ( v(kk,jj,ii) + v(kk,jj+1,ii) ) * 0.5   )
                      ELSE
                         local_pf(i,j,k) = -200.0_wp
                      ENDIF
                   ENDDO
                ENDDO
             ENDDO
          ELSE
             DO  i = 1, mask_size_l(mid,1)
                DO  j = 1, mask_size_l(mid,2)
                   DO  k = 1, mask_size_l(mid,3)
                      kk = mask_k(mid,k)
                      jj = mask_j(mid,j)
                      ii = mask_i(mid,i)
                      IF ( kk .GT. nzb_s_inner(jj,ii) ) THEN
                         local_pf(i,j,k) = 180.0_wp + rad_to_deg * ATAN2(      &
                                 ( u_av(kk,jj,ii) + u_av(kk,jj,ii+1) ) * 0.5,  &
                                 ( v_av(kk,jj,ii) + v_av(kk,jj+1,ii) ) * 0.5   )
                      ELSE
                         local_pf(i,j,k) = -200.0_wp
                      ENDIF
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

!--    wind speed
       CASE ( 'ff' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ff(mask_k(mid,k),             &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ff_av(mask_k(mid,k),             &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

!--    velocity ratio
       CASE ( 'ff_r' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ff_r(mask_k(mid,k),             &
                                             mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ff_r_av(mask_k(mid,k),             &
                                                mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

!--    turbulence intensity
       CASE ( 'ti' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ti(mask_k(mid,k),             &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ELSE
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ti_av(mask_k(mid,k),             &
                                              mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

!--    Output of maximum wind speed of a certain time interval (i.e. 2 min or 10 min max)
       CASE ( 'uvmax' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = uvmax(mask_k(mid,k),                   &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
             uvmax = 0.0_wp  !reset uvmax after every output
          ENDIF

       CASE ( 'ui' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = ui(mask_k(mid,k),                      &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE ( 'vi' )
          IF ( av == 0 )  THEN
            DO  i = 1, mask_size_l(mid,1)
               DO  j = 1, mask_size_l(mid,2)
                  DO  k = 1, mask_size_l(mid,3)
                      local_pf(i,j,k) = vi(mask_k(mid,k),                      &
                                           mask_j(mid,j),mask_i(mid,i))
                   ENDDO
                ENDDO
             ENDDO
          ENDIF

       CASE DEFAULT
          found = .FALSE.


    END SELECT


 END SUBROUTINE user_data_output_mask
