!> @file user_define_netcdf_grid.f90
!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_define_netcdf_grid.f90 1683 2015-10-07 23:57:51Z knoop $
!
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
!
! 1320 2014-03-20 08:40:49Z raasch
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 211 2008-11-11 04:46:24Z raasch
! Former file user_interface.f90 split into one file per subroutine
!
! Description:
! ------------
!> Set the grids on which user-defined output quantities are defined.
!> Allowed values for grid_x are "x" and "xu", for grid_y "y" and "yv", and
!> for grid_z "zu" and "zw".
!------------------------------------------------------------------------------!
 SUBROUTINE user_define_netcdf_grid( variable, found, grid_x, grid_y, grid_z )
 

    USE kinds
    
    USE user

    IMPLICIT NONE

    CHARACTER (LEN=*) ::  grid_x     !< 
    CHARACTER (LEN=*) ::  grid_y     !< 
    CHARACTER (LEN=*) ::  grid_z     !< 
    CHARACTER (LEN=*) ::  variable   !< 

    LOGICAL ::  found   !< 


    SELECT CASE ( TRIM( variable ) )

       CASE ( 'dd', 'dd_xy', 'dd_xz', 'dd_yz' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zu'

       CASE ( 'ff', 'ff_xy', 'ff_xz', 'ff_yz' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zu'

       CASE ( 'ff_r', 'ff_r_xy', 'ff_r_xz', 'ff_r_yz' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zu'

       CASE ( 'ti', 'ti_xy', 'ti_xz', 'ti_yz' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zu'

       CASE ( 'dd_b*_xy', 'ff_b*_xy', 'ff_r_b*_xy', 'ti_b*_xy', 's_b*_xy' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zu1'

       CASE ( 'u_b*_xy' )
          grid_x = 'xu'
          grid_y = 'y'
          grid_z = 'zu1'

       CASE ( 'v_b*_xy' )
          grid_x = 'x'
          grid_y = 'yv'
          grid_z = 'zu1'

       CASE ( 'u2', 'u2_xy', 'v2', 'v2_xy' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zw'

       CASE ( 'ui', 'ui_xy', 'vi', 'vi_xy' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zw'

       CASE ( 'uvgs', 'uvgs_xy', 'uvgs_xy_av')
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zw'

       CASE ( 'uvmax', 'uvmax_xy' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zw'

       CASE ( 'gust11', 'gust21', 'gust31', 'gust41', 'gust51', 'gust61',      &
              'gust12', 'gust22', 'gust32', 'gust42', 'gust52', 'gust62',      &
              'gust13', 'gust23', 'gust33', 'gust43', 'gust53', 'gust63' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zw'

       CASE ( 'ros011', 'ros021', 'ros031', 'ros041', 'ros051', 'ros061',      &
             'ros071', 'ros081', 'ros091', 'ros101', 'ros111', 'ros121',       &
             'ros012', 'ros022', 'ros032', 'ros042', 'ros052', 'ros062',       &
             'ros072', 'ros082', 'ros092', 'ros102', 'ros112', 'ros122',       &
             'ros013', 'ros023', 'ros033', 'ros043', 'ros053', 'ros063',       &
             'ros073', 'ros083', 'ros093', 'ros103', 'ros113', 'ros123',       &
             'ros014', 'ros024', 'ros034', 'ros044', 'ros054', 'ros064',       &
             'ros074', 'ros084', 'ros094', 'ros104', 'ros114', 'ros124',       &
             'ros015', 'ros025', 'ros035', 'ros045', 'ros055', 'ros065',       &
             'ros075', 'ros085', 'ros095', 'ros105', 'ros115', 'ros125',       &
             'ros016', 'ros026', 'ros036', 'ros046', 'ros056', 'ros066',       &
             'ros076', 'ros086', 'ros096', 'ros106', 'ros116', 'ros126' )
          grid_x = 'x'
          grid_y = 'y'
          grid_z = 'zw'

       CASE DEFAULT
          found  = .FALSE.
          grid_x = 'none'
          grid_y = 'none'
          grid_z = 'none'

    END SELECT


 END SUBROUTINE user_define_netcdf_grid

