!> @file user_init.f90
!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_init.f90 1683 2015-10-07 23:57:51Z knoop $
!
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
! 
! 1353 2014-04-08 15:21:23Z heinze
! REAL constants provided with KIND-attribute 
!
! 1320 2014-03-20 08:40:49Z raasch
! revision history before 2012 removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 211 2008-11-11 04:46:24Z raasch
! Former file user_interface.f90 split into one file per subroutine
!
! Description:
! ------------
!> Execution of user-defined initializing actions
!------------------------------------------------------------------------------!
 SUBROUTINE user_init
 

    USE arrays_3d,                                                             &
        ONLY:  qsws, shf

    USE control_parameters
    
    USE grid_variables,                                                        &
        ONLY:  dx, dy

    USE indices
    
    USE kinds
    
    USE netcdf_control
    
    USE pegrid
    
    USE statistics,                                                            &
        ONLY:  rmask, statistic_regions

    USE user

    IMPLICIT NONE

    CHARACTER (LEN=20) :: field_char   !< 

    INTEGER(iwp) ::  id_outflow_l, id_source_out_l
    INTEGER(iwp) ::  i, ii, j
    REAL(wp)     ::  xx, yy

    REAL(wp), DIMENSION(:,:), ALLOCATABLE :: oro_height !< height of orography

!-- variables for gust analysis
    CHARACTER (LEN=4)  :: pos_vms_z, pos_vms_y, pos_vms_x

    INTEGER(iwp) ::  m, n, x, y
    INTEGER(iwp) ::  blx     !:
    INTEGER(iwp) ::  bly     !:
    INTEGER(iwp) ::  bxlg    !:
    INTEGER(iwp) ::  bxrg    !:
    INTEGER(iwp) ::  byng    !:
    INTEGER(iwp) ::  bysg    !:
    INTEGER(iwp) ::  n_total_timesteps

!
!-- Here the user-defined initializing actions follow:
!-- Sample for user-defined output
!    ALLOCATE( u2(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
!    ALLOCATE( ustvst(nzb:nzt+1,nysg:nyng,nxlg:nxrg) );  ustvst = 0.0_wp

!-- Sample for user-defined time series
!-- For each time series quantity you have to give a label and a unit,
!-- which will be used for the NetCDF file. They must not contain more than
!-- seven characters. The value of dots_num has to be increased by the
!-- number of new time series quantities. Its old value has to be store in
!-- dots_num_palm. See routine user_statistics on how to output calculate
!-- and output these quantities.
!     dots_label(dots_num+1) = 'vol_in'  !volume flow at inflow boundary
!     dots_unit(dots_num+1)  = 'm2/s'
!     dots_label(dots_num+2) = 'vol_out' !volume flow at outflow boundary
!     dots_unit(dots_num+2)  = 'm2/s'
! 
!     dots_num_palm = dots_num
!     dots_num = dots_num + 2


!-- TIMESERIES at virtual measurement stations !/home/h/nikkasch or ~
!-------------------------------------------------------------------------------
    ALLOCATE( pos_vms(1:n_vms,1:3) ); pos_vms = 0.0

    OPEN( 600, FILE='VMS_POSITIONS'//coupling_char, STATUS='OLD', &
               FORM='FORMATTED' )

    DO  n = 1, n_vms
       READ( 600, * ) pos_vms(n,1), pos_vms(n,2), pos_vms(n,3)
    ENDDO

    CLOSE( 600 )

    DO  n = 1, n_vms
       WRITE ( pos_vms_z, '(I4.4)' ) pos_vms(n,1)*gust_k_level
       WRITE ( pos_vms_y, '(I4.4)' ) pos_vms(n,2)*gust_k_level
       WRITE ( pos_vms_x, '(I4.4)' ) pos_vms(n,3)*gust_k_level
       dots_label(dots_num+n) = 'uv'//TRIM(pos_vms_z)//"_"//TRIM(pos_vms_y)//"_"//TRIM(pos_vms_x)
       dots_unit(dots_num+n)  = 'm/s'
       dots_label(dots_num+n_vms+n) = 'dd'//TRIM(pos_vms_z)//"_"//TRIM(pos_vms_y)//"_"//TRIM(pos_vms_x)
       dots_unit(dots_num+n_vms+n)  = 'degree'
    ENDDO

    dots_num_palm = dots_num
    dots_num = dots_num + (2*n_vms)



!
!-- Define id_outflow and id_source_out
    IF ( turbulent_outflow )  THEN

       IF ( nxr == nx )  THEN
          id_outflow_l = myidx
       ELSE
          id_outflow_l = 0
       ENDIF
       IF ( collective_wait )  CALL MPI_BARRIER( comm2d, ierr )
       CALL MPI_ALLREDUCE( id_outflow_l, id_outflow, 1, MPI_INTEGER, MPI_SUM, &
                           comm1dx, ierr )

       IF ( NINT( outflow_source_plane / dx ) >= nxl  .AND. &
            NINT( outflow_source_plane / dx ) <= nxr )  THEN
          id_source_out_l = myidx
       ELSE
          id_source_out_l = 0
       ENDIF
       IF ( collective_wait )  CALL MPI_BARRIER( comm2d, ierr )
       CALL MPI_ALLREDUCE( id_source_out_l, id_source_out, 1, MPI_INTEGER, &
                           MPI_SUM, comm1dx, ierr )
    ENDIF

!
!-- Define statistic_region(1:2) as recycling area and outflow area
    IF ( statistic_regions >= 2 ) THEN
       DO  i = nxl, nxr
          xx = i * dx
          IF ( xx < recycling_width ) THEN
             rmask(nys:nyn,i,1) = 1.0_wp
             rmask(nys:nyn,i,2) = 0.0_wp
          ELSEIF ( xx > outflow_source_plane ) THEN
             rmask(nys:nyn,i,1) = 0.0_wp
             rmask(nys:nyn,i,2) = 1.0_wp
          ELSE
             rmask(nys:nyn,i,1) = 0.0_wp
             rmask(nys:nyn,i,2) = 0.0_wp
          ENDIF
       ENDDO
    ENDIF

!
!-- Define statistic_region(3) as same area as mask area
    IF ( statistic_regions >= 3 ) THEN
       DO  i = nxl, nxr
          xx = i * dx
          IF ( xx >= 2900.0_wp .AND. xx <= 7300.0_wp ) THEN
             DO  j = nys, nyn
                yy = j * dy
                IF ( yy >= 500.0_wp .AND. yy <= 5600.0_wp ) THEN
                   rmask(j,i,1) = 1.0_wp
                ELSE
                   rmask(j,i,1) = 0.0_wp
                ENDIF
             ENDDO
          ELSE
             rmask(nys:nyn,i,1) = 0.0_wp
          ENDIF
       ENDDO
    ENDIF

!
!-- Define statistic_region(4) as city area (orography>0.0m)
!   !!! WARNING !!!! OROGRAPHY_DATA is NOT orography but land use!
    IF ( statistic_regions >= 4 ) THEN

       ALLOCATE ( oro_height(0:ny,0:nx) )

       DO  ii = 0, io_blocks-1
          IF ( ii == io_group )  THEN
!
!--          Orography information, used to distinguish between city area
!--          and sea area
             OPEN( 91, FILE='OROGRAPHY_DATA'//coupling_char, STATUS='OLD', &
                   FORM='FORMATTED', ERR=10 )
             DO  j = ny, 0, -1
                READ( 91, *, ERR=11, END=11 )  ( oro_height(j,i), i = 0,nx )
             ENDDO

             GOTO 12
       
10             message_string = 'file OROGRAPHY'//coupling_char//' does not exist'
             CALL message( 'user_init', 'TG0101', 1, 2, 0, 6, 0 )

11             message_string = 'errors in file OROGRAPHY_DATA'//coupling_char
             CALL message( 'user_init', 'TG0102', 1, 2, 0, 6, 0 )

12             CLOSE( 91 )

          ENDIF
#if defined( __parallel ) && ! defined ( __check )
          CALL MPI_BARRIER( comm2d, ierr )
#endif
       ENDDO

       DO  i = nxl, nxr
          DO  j = nys, nyn
             IF ( oro_height(j,i) .GT. 0.0_wp ) THEN
                rmask(j,i,4) = 1.0_wp
             ELSE
                rmask(j,i,4) = 0.0_wp
             ENDIF
          ENDDO
       ENDDO

       !CALL exchange_horiz_2d( rmask(j,i,4) )

    ENDIF

!
!-- Over topography surface_heatflux is replaced by wall_heatflux(0)
!-- (this is done in init_3d_model but will be overwritten in read_3d_binary)
    IF ( constant_heatflux )  THEN
       IF ( TRIM( topography ) /= 'flat' )  THEN
          DO  i = nxlg, nxrg
             DO  j = nysg, nyng
                IF ( nzb_s_inner(j,i) /= 0 )  THEN
                   shf(j,i) = wall_heatflux(0)
                ENDIF
             ENDDO
          ENDDO
       ENDIF
    ENDIF


!
!-- Define scalar volume source inside statisic_region(4)
!-- oro_height consists of a flag field, which is >0 inside the city and >1 at
!-- building positions
    IF ( statistic_regions >= 4 .AND. emission_rate > 0.0_wp ) THEN
       ALLOCATE( s_emission_rate(nys:nyn,nxl:nxr) )
       s_emission_rate = 0.0_wp
       DO  i = nxl, nxr
          DO  j = nys, nyn
             IF ( oro_height(j,i) .EQ. 1.0_wp )  &
                s_emission_rate(j,i) = emission_rate
          ENDDO
       ENDDO
    ENDIF

    IF ( ALLOCATED( oro_height ) )  DEALLOCATE( oro_height )

    qsws = 0.0_wp  !be sure that there is no scalar surface flux

!
!-- Define output arrays
    ALLOCATE( ff(nzb:nzt+1,nysg:nyng,nxlg:nxrg),    &
              ff_r(nzb:nzt+1,nysg:nyng,nxlg:nxrg),  &
              ti(nzb:nzt+1,nysg:nyng,nxlg:nxrg)     )

    ff = 0.0_wp ; ff_r = 0.0_wp ; ti = 0.0_wp

!
!-- Set fill values inside topography
    DO  i = nxlg, nxrg
       DO  j = nysg, nyng
          ff(nzb:nzb_s_inner(j,i),j,i)   = -1.0_wp
          ff_r(nzb:nzb_s_inner(j,i),j,i) = -1.0_wp
          ti(nzb:nzb_s_inner(j,i),j,i)   = -1.0_wp
       ENDDO
    ENDDO


!-- TIMESERIES of horizontal wind direction
    ALLOCATE( dd(nzb:nzt+1,nysg:nyng,nxlg:nxrg) ) 
    dd = 0.0_wp

!-- windroses
    ALLOCATE( char_ndd(ndd) )
    ALLOCATE( char_nff(nff) )

    DO  n = 1, ndd
       WRITE( char_ndd(n), '(I2.2)' ) n
    ENDDO

    DO  m = 1, nff
       WRITE( char_nff(m), '(I1)' ) m
    ENDDO

    ALLOCATE(windspeed_categ(nff))
    windspeed_categ = (/ 0.75_wp, 1.50_wp, 2.25_wp, 3.00_wp, 3.75_wp /)

! 
!-- 3D gust online analysis
    ALLOCATE( uvmax(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )

    namplit = 6_iwp
    ninterv = 3_iwp

    ALLOCATE( char_interv(ninterv), char_amplit(namplit) )

    DO  n = 1, ninterv
       WRITE( char_interv(n), '(I1)' ) n
    ENDDO

    DO  m = 1, namplit
       WRITE( char_amplit(m), '(I1)' ) m
    ENDDO

    uvmax                              = 0.0_wp
!-------------------------------------------------------------------------------


!-- CROSS-SECTIONS of horizontal wind
!-------------------------------------------------------------------------------
    ALLOCATE( uvgs(nzb:nzt+1,nysg:nyng,nxlg:nxrg) ) 
    uvgs = 0.0_wp
    ALLOCATE( ui(nzb:nzt+1,nysg:nyng,nxlg:nxrg) ) 
    ALLOCATE( vi(nzb:nzt+1,nysg:nyng,nxlg:nxrg) ) 
    ui = 0.0_wp
    vi = 0.0_wp
    ALLOCATE( u2(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    ALLOCATE( v2(nzb:nzt+1,nysg:nyng,nxlg:nxrg) )
    u2 = 0.0_wp
    v2 = 0.0_wp


 END SUBROUTINE user_init
