!> @file user_module.f90
!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: user_module.f90 1683 2015-10-07 23:57:51Z knoop $
!
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable 
!
! 1320 2014-03-20 08:40:49Z raasch
! kind-parameters added to all INTEGER and REAL declaration statements, 
! kinds are defined in new module kinds, 
! old module precision_kind is removed,
! revision history before 2012 removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements 
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! Revision 1.1  1998/03/24 15:29:04  raasch
! Initial revision
!
!
! Description:
! ------------
!> Declaration of user-defined variables. This module may only be used
!> in the user-defined routines (contained in user_interface.f90).
!------------------------------------------------------------------------------!
 MODULE user
 

    USE kinds

    IMPLICIT NONE

    INTEGER(iwp) ::  dots_num_palm   !< 
    INTEGER(iwp) ::  user_idummy     !< 
    
    LOGICAL ::  user_defined_namelist_found = .FALSE.   !< 
    
    REAL(wp) ::  user_rdummy   !< 

!
!-- Turbulent outflow condition:
    INTEGER(iwp) ::  id_outflow = 0,    &!< idx of outflow PEs
                     id_source_out = 0   !< idx of PEs containing the outflow
                                         !  source plane

    REAL(wp) ::  outflow_source_plane = 0.0  !< x position (m) of yz plane where
                                             !  outflow values are copied from
                                             !  in case of turbulent_outflow

    LOGICAL ::  fix_mg_levels = .FALSE.,        &!< fixes the mg_levels to a maximum of 2 (does not work for 'multigrid_fast')
                mean_outflow = .FALSE.,         &!< turns on mean profiles at outflow boundary
                turbulent_outflow = .FALSE.,    &!< turns on turbulent ouflow boundary
                turbulent_outflow_mean = .FALSE. !< use mean profiles at outflow boundary instead of instantaneous values

!
!-- Temperature damping at the inflow boundary
    REAL(wp) :: pt_damping_height = 9999999.9_wp !< height (m) from where on pt damping starts

    REAL(wp), DIMENSION(:,:), ALLOCATABLE :: ptdf_xz !<damping factor for pt in xz-direction

!
!-- Scalar
    REAL(wp) ::  emission_rate = 0.0_wp !< input parameter defining the strength of the scalar volume source

    REAL(wp), DIMENSION(:,:), ALLOCATABLE ::  &
          s_emission_rate      !< scalar emission rate of volume source

!
!-- User Output
    REAL(wp) :: vol_flow_in,  &!< volume flow at the inflow boundary
                vol_flow_out   !< volume flow at the outflow boundary

    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  &
          ff, ff_av,      &!< wind speed (3D)
          ff_r, ff_r_av,  &!< velocity ratio ( ff / ff(nzt) )
          ti, ti_av        !< turbulence intensity

!
!-- for windrose calculation
    CHARACTER (LEN = 2), DIMENSION(:), ALLOCATABLE ::  &
          char_ndd, char_nff ! zur benennung der windrosenfelder

    INTEGER(iwp) :: n_vms, & ! Anzahl virtueller messstationen in VMS_POSITIONS
                    ndd = 12_iwp,   & ! Anzahl Windrichtungssektoren
                    nff = 6_iwp       ! Anzahl Windgeschwindigkeitsklassen
    INTEGER(iwp), DIMENSION(:), ALLOCATABLE ::  &
          windspeed_categ    ! Windklassen für windrose 

    INTEGER(iwp), DIMENSION(:,:), ALLOCATABLE ::  &
          pos_vms            ! k,j,i-Positionen virtueller messtration

    INTEGER(iwp), DIMENSION(:,:,:,:,:), ALLOCATABLE ::  &
          n_ff               ! anzahl der windgeschwindigkeiten und -richtungen

!
!-- for calculation of horizontal windspeed (3d arrays)
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  &
          ui, ui_av, &     ! u(zw)
          uvgs, uvgs_av, & ! ff(zw)
          vi, vi_av, &     ! v(zw)
          dd               ! dd(zw)

!
!-- for calculation of eddy covariance momentum fluxes
    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE ::  &
          u2, u2_av, v2, v2_av  ! u^2(zw), v^2(zw)

!
!-- for calculation of ngust via amplitude and interval
    INTEGER(iwp) :: gust_k_level,    &
                    mta_interval,    &
                    namplit=6,       &
                    ninterv=3

    CHARACTER (LEN = 2), DIMENSION(:), ALLOCATABLE ::  &
          char_interv, char_amplit ! zu benennung der ausgabevariablen

!
!-- for calculation of uv_max every 10 (2) minutes interval, depending on dt_domask 
    INTEGER(KIND=8) :: ind_time, n_uvmax_int

    REAL(wp), DIMENSION(:,:,:), ALLOCATABLE :: uvmax

    SAVE

 END MODULE user
